;*****************************************************
;*      Full-screen memory editor for PB-1000/C      *
;*       (C)2007 Miyura (miyura321@ybb.ne.jp)        *
;*****************************************************

PB1000:	EQU	1
PB1000C:EQU	2
MODEL:	EQU	PB1000

CHEX1:	EQU	&H0160		;ascii -> binary

#if MODEL=PB1000
CROFF:	EQU	&H913F		;disable cursor brink
KYIN: 	EQU	&HE8B9		;key scan
#else
CROFF:	EQU	&H91F0		;disable cursor brink
KYIN: 	EQU	&HEAAD		;key scan
#endif

EDTOP:	EQU	&H6100		;EDTOP buffer
KYSTA:	EQU	&H68D2		;key input status


	ORG	EDTOP + 108
	START	START

START:	LDW	$12,&H0008	;
	XRM	$14,$14,8	;initialize resistors
	XRM	$22,$22,8	;
	CAL	CROFF		;cursor off
;
;	clear EDTOP
;
	PRE	IX,EDTOP
	LD	$0,&H7F
STA01:  LD	$1,$0
	AN	$1,&H1F
	SBC	$1,&H1D		;check ":" position
	JR	NZ,STA02	;
	LD	$2,":",STA03
STA02:	LD	$2," "
STA03:	ST	$2,(IX+$0)
	SB	$0,$30
	JR	NC,STA01

	CAL	SETBK		;set bank and dump 32 bytes
	PST	IB,$14		;set IB resister

RETUN:	LDW	$0,RETUN-1	;set return address
	PHSW	$1		;
;
;	set cursor
;
CSR:	LD	$0,$18		;$0  <- $18
	LD	$16,$0		;$16 <- $18

	SBC	$28,$31		;check input mode flag
	JR	NZ,CSR01	;
;
;	address input mode
;
	AN	$0,&H03		;$0  =  X
	AN	$16,&H0C	;$16 =  4 * Y
	DIU	$16		;$16 = 64 * Y
	BID	$16		;$16 = 32 * Y
	OR	$16,$0		;$16 = 32 * Y + X
	AD	$16,$30,KEY	;$16 = $16 + 1
;
;	data input mode
;
CSR01:	AN	$0,&H07		;$0 =  X
	AN	$16,&H18	;$16 =  8 * Y
	BIU	$16		;$16 = 16 * Y
	BIU	$16		;$16 = 32 * Y
	LD	$1,$0		;$1  =  X
	BIU	$1		;$1  =  2 * X
	AD	$1,$0		;$1  =  3 * X
	OR	$16,$1		;$16 = 32 * Y + 3 * X
	AD	$16,6		;$16 = $1 + 6
	AD	$16,$29		;$16 = $1 + cursor digit flag

KEY:	GST	UA,$15		;save UA resistor
	PST	IB,&HA0		;reset IB resistor
	CAL	@KYIN		;check input key
;
;	convert hexkeys to A-F
;
	PRE	IX,HEXKY
	LD	$1,"A"
KEY01:	LDI	$2,(IX+$31)
	SBC	$2,$0
	JR	Z,KEY02
	SBC	$1,"F"
	JR	Z,KEY03
	AD	$1,$30,KEY01
KEY02:	LD	$0,$1

KEY03:	CAL	CHEX1
	PST	UA,$15		;load UA resistor 
	PST	IB,$14		;set IB resistor
	JR	C,WRITE
;
;	load jump address of sub routines
;
	PRE	IX,JPTBL
	LD	$5,25		;number of sub routines
KEY04:	LDIM	$1,(IX+$31),3
	SBC	$1,$0
	JR	Z,KEY05
	SB	$5,$30
	JR	NZ,KEY04
	JR	KEY
KEY05:	JP	$2		;jump to sub routine
;
;	input 0-9,A-F keys
;
WRITE:	SBC	$28,$31		;check input mode flag
	JR	NZ,WRT03	;
;
;	input address
;
	LD	$1,$31		;$0 = key code (0-9,A-F)
	LDW	$2,&H000F	;$1 = 00h
	LD	$4,$18		;$2 = 0Fh (mask)
	AN	$4,&H03		;$3 = 00h
WRT01:	SBC	$4,3		;$4 = X address (0-3)
	JR	Z,WRT02
	DIUM	$0,4
	AD	$4,$30,WRT01	;$4 = $4 + 1

WRT02:	LDW	$4,$18		;
	AN	$4,&H0C		;$4 = Y address * 4
	BIU	$4		;$4 = $4 * 2
	ADW	$20,$4
	INVW	$2
	ANW	$20,$2		;clear the digit
	ORW	$20,$0		;insert input number
	SBW	$20,$4

	SBC	$18,15		;check scroll
	JR	Z,RIG01		;
	AD	$18,$30
	JP	DUMP
;
;	write data
;
WRT03:	PRE	IZ,$20		;load binary data
	LDD	$1,(IZ+$18)	;
	SBC	$29,$31		;check cursor digit flag
	JR	NZ,WRT04	;
	DIU	$0		;upper digit
	AN	$1,&H0F,WRT05	;lower digit
WRT04:	AN	$1,&HF0		;
WRT05:	OR	$1,$0		;merge
	ST	$1,(IZ+$31)	;write input data
	CAL	DUMP1		;1 byte dump
	JR	RIG02		;
;
;	move cursor to right
;
RIGHT:	SBC	$28,$31		;check input mode flag
	JR	NZ,RIG02	;
;
;	address input mode
;
	AD	$18,$30		;$18 = $18 + 1
	SBC	$18,16		;
	RTN	C		;check scroll
RIG01:	LD	$18,12,ROLUP	;top of line 3rd.
;
;	data input mode
;
RIG02:	XR	$29,$30		;check cursor digit flag
	RTN	NZ		;
RIG03:	AD	$18,$30		;$18 = $18 + 1
	SBC	$18,32		;
	RTN	C		;scroll check
RIG04:	LD	$18,24,ROLUP	;top of line 3rd.
;
;	move cursor to left
;
LEFT:	SBC	$28,$31		;check input mode flag
	JR	NZ,LFT01	;
;
;	address input mode
;
	LD	$0,3,LFT02
;
;	data input mode
;
LFT01:	XR	$29,$30		;change cursor digit flag
	RTN	Z		;
	LD	$0,7
LFT02:	SB	$18,$30		;$18 = $18 - 1
	RTN	NC		;check scroll
	LD	$18,$0,ROLDN	;end of line 1st.
;
;	up cursor
;
UP:	SBC	$28,$31		;check input mode flag
	JR	NZ,UP01		;
;
;	address input mode
;
	LD	$0,4,UP02	;$18 = $18 - 4
;
;	data input mode
;
UP01:	LD	$0,8		;$18 = $18 - 8
UP02:	SB	$18,$0		;
	RTN	NC		;check scroll
	AD	$18,$0		;
ROLDN:	SBW	$20,$12,DUMP	;roll down (8byte)
;
;	down cursor
;
DOWN:	SBC	$28,$31		;check input mode flag
	JR	NZ,DWN01	;
;
;	address input mode
;
	SBC	$18,12		;check scroll
	JR	NC,ROLUP	;
	AD	$18,4		;$18 = $18 + 4
	RTN
;
;	data input mode
;
DWN01:	SBC	$18,24		;check scroll
	JR	NC,ROLUP	;
	AD	$18,8		;$18 = $18 + 8
	RTN
;
;	reset cursor position
;
CURRE:	SBC	$28,$31		;check input mode flag
	RTN	Z		;
	LDW 	$0,$18
	AN	$0,&H07
	AN	$18,&H18
	ADW	$20,$0,DUMP
;
;	reset address
;
ADRRE:	LD	$0,$20
	AN	$20,&HF8

	SBC	$28,$31		;check input mode flag
	JR	Z,DUMP		;

	SB	$0,$20
	AD	$18,$0
	SBC	$18,32		;scroll check
	JR	C,DUMP		;
	SB	$18,8		;$18 = $18 - 8
ROLUP:	ADW	$20,$12,DUMP	;roll up (8byte)
;
;	up screen (32byte)
;
SCLUP:	LDW	$0,32
	ADW	$20,$0,DUMP
;
;	down screen (32byte)
;
SCLDN:	LDW	$0,32
	SBW	$20,$0,DUMP
;
;	change bank
;
BANKC:	AD	$17,$30		;bank + 1
	AN	$17,&H03	;$17 = 4 -> 0
SETBK:	LD	$0,$17
	DIU	$0		;x 8
	BIU	$0		;x 2
	BIU	$0		;x 2
	PST	UA,$0,DUMP	;set bank of IZ
;
;;	change IB resitor
;
EXIB:	XR	$14,&H80	;set IB resistor
	PST	IB,$14,DUMP	;
;
;	exchange address
;
EXADR:	SBC	$28,$31		;check input mode flag
	RTN	Z		;
	LDM	$0,$18,4	;$18,19 <-> $22,23
	LDM	$18,$22,4	;$20,21 <-> $24,25
	LDM	$22,$0,4	;
	CAL	SAS01
;
;	dump 32bytes binary data
;
DUMP:	PRE	IZ,$20
	PRE	IX,EDTOP
	LD	$10,4		;set line counter

DMP01:	LD	$0,$17		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STI	$1,(IX+$31)	;output bank number

	GRE	IZ,$3		;$3,4 = address
	LD	$0,$4		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$31)	;output address (upper)

	LD	$0,$3		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$31)	;output address (lower)

	LDW	$6,&H0800	;reset counter
DMP02:	LDI	$0,(IZ+$31)	;load binary data
	AD	$6,$0		;add check-sum counter
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$30)	;output data
	SB	$7,$30		;counter - 1
	JR	NZ,DMP02	;

	LD	$0,$6		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$30)	;output check-sum

	SB	$10,$30		;line counter - 1
	JR	NZ,DMP01	;
;
;	display data on LCD (character mode)
;
	PRE	IX,EDTOP	;left side
	LD	$0,&H83		;set character output mode
	LDW	$1,&H03DE	;x=222, y=3 (for 1 line shift)
	CAL	DMP03

	LDD	$0,(IX+&H10)	;right side (IX=EDTOP+&H10)
	LD	$0,&H93		;set character output mode
DMP03:	PPO	&HC7		;set LCD command
	STLM	$0,3		;
	PPO	&HC6
	LD	$0,$31		;reset counter
DMP04:	LD	$3,(IX+$0)	;load ascii data
	CAL	SWAP		;swap $3 and output to LCD
	AD	$0,$30		;counter + 1
	ANC	$0,&H10		;
	JR	Z,DMP05		;
	AD	$0,&H10		;add offset
DMP05:	SBC	$0,&H80		;check end
	JR	NZ,DMP04	;
	RTN
;
;	calculate relative jump
;
JR:	SBC	$28,$31		;check input mode flag
	RTN	Z		;

	CAL	SASET

	PRE	IZ,$26
	LD	$0,(IZ+$31)
	LD	$1,$31
	ANC	$0,&H80		;check jump direction
	JR	NZ,JR01		;
;
;	positive jump
;
	AD	$18,$0
	SBC	$18,&H20	;check scroll
	RTN	C		;

	LDW	$0,$18
	AN	$18,&H07
	OR	$18,&H18
	AN	$0,&HF8
	SB	$0,&H18
	ADW	$20,$0,@DUMP
;
;	negative jump
;
JR01:	AN	$0,&H7F
	SB	$18,$0		;check scroll
	RTN	NC		;

	LD	$0,$18
	AN	$18,&H07
	INV	$0
	AN	$0,&HF8
	AD	$0,&H08
	SBW	$20,$0,@DUMP
;
;	back space
;
BS:	SBC	$28,$31		;check input mode flag
	RTN	Z		;
	CAL	LFT01
;
;	delete data
;
DEL:	SBC	$28,$31		;check input mode flag
	RTN	Z		;

	CAL	DIFF
	RTN	C		;if cursor address exceeds sub address return

DEL01:	PRE	IZ,$2
	JR	Z,DEL05

	SBC	$29,$31		;check cursor digit flag
	JR	Z,DEL02		;

	LDW	$0,(IZ+$31)
	DID	$1
	AN	$0,&HF0
	OR	$1,$0,DEL03

DEL02:	LD	$1,(IZ+$31)
	LD	$0,(IZ+$30)
	DIUW	$0
DEL03:	STI	$1,(IZ+$31)
	SBW	$4,$30
	JR	NZ,DEL02

DEL04:	LD	$0,(IZ+$31)
	DIU	$0,INS05

DEL05:	SBC	$29,$31		;check cursor digit flag
	JR	Z,DEL04		;
	JR	INS02
;
;	insert data
;
INS:	SBC	$28,$31		;check input mode flag
	RTN	Z		;

	CAL	DIFF
	RTN	C		;if cursor address exceeds sub address return

	PRE	IZ,$26
	JR	Z,INS02

INS01:	LD	$0,(IZ+$31)
	LDD	$1,(IZ-$30)
	DIDW	$1
	ST	$0,(IZ+$30)
	SBW	$4,$30
	JR	NZ,INS01

INS02:	LD	$0,(IZ+$31)

	SBC	$29,$31		;check cursor digit flag
	JR	Z,INS04		;

INS03:	AN	$0,&HF0,INS05

INS04:	DID	$0
INS05:	ST	$0,(IZ+$31)
@DUMP:	JP	DUMP
;
;	line delete data
;
LDEL:	CAL	DIFF

	SBC	$28,$31		;check input mode flag
	RTN	Z		;

	LDW	$4,8
	AN	$0,&H07
	SB	$4,$0,CLM00
;
;	clear data
;
CLM:	CAL	DIFF
	RTN	C		;if cursor address > sub address return

	SBC	$28,$31		;check input mode flag
	RTN	Z		;

	ADW	$4,$30
CLM00:	PRE	IZ,$2		;set start address

	SBC	$29,$31		;check cursor digit flag
	JR	Z,CLM01		;

	LD	$0,(IZ+$31)	;
	AN	$0,&HF0		;clear lower digit
	STI	$0,(IZ+$31)	;
	JR	CLM02		;

CLM01:	STI	$31,(IZ+$31)	;store null
CLM02:	SBW	$4,$30		;counter - 1
	JR	NZ,CLM01
	JR	@DUMP
;
;	calculate relative jump offset
;
OFFST:	SBC	$28,$31		;check input mode flag
	RTN	Z		;

	LDW	$0,$20
	ADW	$0,$18
	PRE	IZ,$0
	LD	$4,&H80
	SBW	$0,$26
	JR	NC,OFF01

	CMPW	$0
	LD	$4,$31
OFF01:	LDW	$2,&H0080
	SBCW	$0,$2
	RTN	NC

	OR	$4,$0
	ST	$4,(IZ+$31)
	JR	@DUMP
;
;	move cursor to home position
;
HOME:	LD	$18,$31,EXC00
;
;	move cursor to line top
;
LNTOP:	SBC	$28,$31		;check mode
	JR	NZ,LNT01	;
;
;	address input mode
;
	AN	$18,&H0C
	RTN
;
;	data input mode
;
LNT01:	AN	$18,&H18,EXC00
;
;	move cursor to line end
;
LNEND:	SBC	$28,$31		;check input mode flag
	JR	NZ,LNE01	;
;
;	address input mode
;
	OR	$18,&H03
	RTN
;
;	data input mode
;
LNE01:	OR	$18,&H07
	LD	$29,$30
	RTN
;
;	exchange cursor position
;
EXCSR:	XR	$28,$30
	JR	NZ,EXC01

	AN	$18,&H18
	BID	$18
EXC00:	LD	$29,$31
	RTN

EXC01:	AN	$18,&H0C
	BIU	$18
	RTN
;
;	output ascii character
;
ASCII:	LDW	$7,&H8320	;set character output mode
	LDW	$9,&H02DE	;x=222, y=2 (for 1 line shift)
	JR	OUTDT
;
;	output bitmap data
;
BITMP:	LDW	$7,&H8210	;set binary output mode
	LDW	$9,&H0300	;x=0, y=3

OUTDT:	PRE	IZ,$20
	CAL	OUTSB		;left side
	OR	$8,&H10		;
	CAL	OUTSB		;right side

	PST	IA,&H5		;
OUT01:	GRE	KY,$0		;check key hold
	SBC	$7,$0		;
	JR	Z,OUT01		;erase output data
	JP	DUMP

OUTSB:	PPO	&HC7		;
	STLM	$8,3		;set LCD command
	PPO	&HC6		;
	LD	$6,16		;set counter
	SBC	$7,&H20		;check binary/ascii
	JR	Z,OUT03		;
OUT02:	LDIM	$0,(IZ+$31),6	;load binary data
	STLM	$0,6		;output to LCD
	SB	$6,$30		;counter - 1
	JR	NZ,OUT02
	RTN
OUT03:	LDI	$3,(IZ+$31)	;load data
	CAL	SWAP		;swap $3 and output to LCD
	SB	$6,$30		;counter - 1
	JR	NZ,OUT03
	RTN
;
;	set sub address
;
SASET:	LDM	$22,$18,4

	SBC	$28,$31		;check input mode flag
	JR	NZ,SAS01	;

	AN	$22,&H0C
	BIU	$22
SAS01:	LDW	$26,$24
	ADW	$26,$22
	RTN
;
;	calculate distance from cursor address to sub address
;
DIFF:	LDM	$0,$18,4
	ADW	$2,$0
	LDW	$4,$26
	SBW	$4,$2
	RTN
;
;	dump 1 byte binary data
;
DUMP1:	PRE	IZ,$20		;load binary data
	LD	$1,(IZ+$18)	;
	SBC	$29,$31		;check cursor digit
	JR	NZ,DMP06	;
	DID	$1,DMP07
DMP06:	AN	$1,&H0F
DMP07:	CAL	CNA02		;binary -> ascii ($1)

	LD	$10,$16

	PRE	IX,EDTOP	;$1 -> text buffer
	ST	$1,(IX+$10)	;
	CAL	DMP09
;
;	check-sum area
;
	PRE	IZ,$20
	LD	$0,$18
	AN	$0,&H18
	LD	$1,7		;set counter
	LDI	$0,(IZ+$0)
DMP08:	LDI	$2,(IZ+$31)
	AD	$0,$2		;add check-sum counter
	SB	$1,$30		;counter - 1
	JR	NZ,DMP08
	CAL	CNASC		;$0 binary -> ascii $0,$1

	OR	$10,&H1F	;set cursor pointer to check-sum area
	SB	$10,$30		;

	PRE	IX,EDTOP	;$0,$1 -> EDTOP buffer
	STW	$0,(IX+$10)	;
	CAL	DMP10		;output $0 (upper digit)

	AD	$10,$30		;

DMP09:	LD	$0,$1		;output $1 (lower digit)
DMP10:	LD	$8,&H83		;set command
	CAL	LCDXY		;calculate X,Y address
	LD	$3,$0
	CAL	SWAP		;swap and output to LCD
	RTN
;
;	convert binary to ascii
;
CNASC:	LD	$1,$0
	DID	$0		;upper digit
	AD	$0,"0"		;
	SBC	$0,&H3A		;
	JR	C,CNA01		;check 0-9 or A-F
	AD	$0,7		;
CNA01:	AN	$1,&H0F		;lower digit
CNA02:	AD	$1,"0"		;
	SBC	$1,&H3A		;
	RTN	C		;check 0-9 or A-F
	AD	$1,7		;
	RTN
;
;	swap $3 and output to LCD
;
SWAP:	LD	$4,$3		;
	DIU	$4		;swap digit
	DID	$3		;
	OR	$3,$4		;
	STL	$3		;output character to LCD
	RTN
;
;	input key
;
@KYIN:	CAL	CSRST		;erase cursor
	LDW	$0,KYSTA	;key repeat on
	ST	&H06,($0)	;
	CAL	KYIN		;key scan
;
;	cursor set/reset
;
CSRST:	LD	$10,$16
	LD	$8,&H22		;xor mode
	CAL	LCDXY		;calculate X-Y address
	STL	$30		;
	STL	$30		;
	STL	$30		;set cursor bar
	STL	$30		;
	STL	$30		;
	STL	$30		;
	RTN
;
;	calulate LCD X-Y address
;
LCDXY:	LD	$4,$10		;$4 : Y position
	DID	$4		;$4 = $4 /  8
	BID	$4		;$4 = $4 / 16

	LD	$6,$8		;$6 : binary/ascii flag
	AN	$6,$30		;
	JR	Z,LCD00		;

	LD	$5,$10		;check special position
	AN	$5,&H1F		;
	SBC	$5,&H18		;x=24
	JR	Z,LCD01		;
	SBC	$5,&H10		;x=16
	JR	Z,LCD02		;

LCD00:	LD	$3,$10		;$3 : X position within the block
	AN	$3,&H07		;
	LD	$5,$3		;
	BIU	$5		;$5<- $3 *  2
	AD	$3,$5		;$3 = $3 *  3
	BIU	$3		;$3 = $3 *  6
	SB	$3,$6		;$3 = $3 - offset
	BIU	$3		;$3 = $3 * 12

	LD	$5,$10		;$5 = check block 1/2
	AN	$5,&H08		;
	DIU	$5		;$5 = $5 *  8

	OR	$3,$5,LCD03	;$3 = $3 + $5

LCD01:	LD	$3,&H5E,LCD03	;modify X address for x=24

LCD02:	LD	$3,&HDE		;
	SB	$4,$30		;modify X,Y address for x=16
	JR	NC,LCD03	;
	LD	$4,3

LCD03:	LD	$2,$10		;$2 : LCD command
	AN	$2,&H10		;check LCD1/LCD2
	OR	$2,$8		;
	PPO	&HC7		;
	STLM	$2,3		;set LCD command
	PPO	&HC6		;
	RTN
;
;	jump address table
;
JPTBL:	DB	&H1C		;""
	DW	RIGHT

	DB	&H1D		;""
	DW	LEFT

	DB	&H1E		;""  
	DW	UP

	DB	&H1F		;""
	DW	DOWN

	DB	&H02		;LINE TOP
	DW	LNTOP

	DB	&H06		;LINE END
	DW	LNEND

	DB	&H01		;ROLL UP
	DW	SCLUP

	DB	&H10		;ROLL DOWN
	DW	SCLDN

	DB	&H0B		;HOME
	DW	HOME

	DB	" "		;SPACE
	DW	EXCSR

	DB	&H08		;BS
	DW	BS

	DB	&H11		;DEL
	DW	DEL

	DB	&H12		;INS
	DW	INS

#if MODEL=PB1000
	DB	&H05		;L.DEL (SHIFT + BS)
#else
	DB	"_"		;_     (SHIFT + BS)
#endif
	DW	LDEL

	DB	&H0C		;CLS
	DW	CLM

	DB	&H0D		;EXE
	DW	SASET

	DB	&HF8		;ENG
	DW	ADRRE

	DB	&HF9		;SHIFT + ENG
	DW	CURRE

	DB	&HF2		;ANS
	DW	EXADR
 
	DB	"."		;"."
	DW	BANKC

	DB	"O"		;"O"
	DW	OFFST

	DB	"J"		;"J"
	DW	JR

	DB	"Z"		;"Z"
	DW	ASCII

	DB	"X"		;"X"
	DW	BITMP

	DB	"^"		;"^"
	DW	EXIB
;
;	hexkey data table
;
HEXKY:	DB	"()*/+-"