;*********************************************************************
;*            Full-screen memory editor for FX-870P/VX-3             *
;*               (C)2007 Miyura (miyura321@ybb.ne.jp)                *
;*********************************************************************

FX870P:	EQU	1		;FX-870P
VX3:	EQU	2		;VX-3

MODEL:	EQU	FX870P

#if MODEL=FX870P
CHEX1:	EQU	&H009D		;ascii -> binary
CROFF:	EQU	&H032E		;disable cursor brink
KYIN:	EQU	&H03A4		;key scan
BIOS:	EQU	&H5321		;call BIOS(BANK0) and return to BANK1

KYSTA:	EQU	&H1114		;key input status
EDTOP:	EQU	&H113B		;EDTOP buffer

#else
CHEX1:	EQU	&H0125		;ascii -> binary
CROFF:	EQU	&H1FAE		;disable cursor brink
KYIN:	EQU	&H03D1		;key scan
BIOS:	EQU	&H5CA7		;call BIOS(BANK0) and return to BANK1

KYSTA:	EQU	&H621B		;key input status
EDTOP:	EQU	&H6242		;EDTOP buffer
#endif


	ORG	EDTOP + 100
	START	START

START:	LDW	$14,&H0008	;
	LD	$16,&H20	;initialize resistors
	XRM	$17,$17,8	;
	LDW	$28,CROFF	;cursor off
	CAL	RCS		;
;
;	clear EDTOP
;
	PRE	IX,EDTOP
	LD	$0,&H7F
	LDW	$2," :"
STA01:  LD	$1,$0
	AN	$1,&H1F
	SBC	$1,&H1D		;check ":" position
	JR	NZ,STA02	;
	ST	$3,(IX+$0)
	JR	STA03
STA02:	ST	$2,(IX+$0)
STA03:	SB	$0,$30
	JR	NC,STA01

	CAL	SETBK		;set bank and dump 32 bytes

RETUN:	LDW	$0,RETUN-1	;set return address
	PHSW	$1
;
;	set cursor pointer
;
CSR:	LD	$0,$18		;$0  <- $18
	LD	$26,$0		;$26 <- $18

	SBC	$22,$31		;check input mode flag
	JR	NZ,CSR01	;
;
;	address input mode
;
	AN	$0,&H03		;$0  =  X
	AN	$26,&H0C	;$26 =  4 * Y
	DIU	$26		;$26 = 64 * Y
	BID	$26		;$26 = 32 * Y
	OR	$26,$0		;$26 = 32 * Y + X
	AD	$26,$30,KEY	;$26 = $1 + 1
;
;	data input mode
;
CSR01:	AN	$0,&H07		;$0  =  X
	AN	$26,&H18	;$26 =  8 * Y
	BIU	$26		;$26 = 16 * Y
	BIU	$26		;$26 = 32 * Y
	LD	$1,$0		;$1 =  X
	BIU	$1		;$1 =  2 * X
	AD	$1,$0		;$1 =  3 * X
	OR	$26,$1		;$26 = 32 * Y + 3 * X
	AD	$26,6		;$26 = $26 + 6
	AD	$26,$23		;$26 = $26 + cursor digit flag
;
;	check input key
;
KEY:	GST	UA,$25		;save UA resistor
	CAL	@KYIN		;input key scan
	LDW	$28,CHEX1	;convert ascii -> binary
	CAL	RCS		;
	PST	UA,$25		;load UA resistor
	JR	C,WRITE		;check 0-9,A-F key
;
;	load jump address of sub routines
;
	PRE	IX,JPTBL
	LD	$5,13		;number of sub routines
KEY01:	LDIM	$1,(IX+$31),3
	SBC	$1,$0
	JR	Z,KEY02
	SB	$5,$30
	JR	NZ,KEY01
	JR	KEY
KEY02:	JP	$2		;jump to sub routine
;
;	input 0-9,A-F keys
;
WRITE:	SBC	$22,$31		;check input mode flag
	JR	NZ,WRT03	;
;
;	input address
;
	LD	$1,$31		;$0 = key code (0-9,A-F)
	LDW	$2,&H000F	;$1 = 00h
	LD	$4,$18		;$2 = 0Fh (mask)
	AN	$4,&H03		;$3 = 00h
WRT01:	SBC	$4,3		;$4 = X address (0-3)
	JR	Z,WRT02
	DIUM	$0,4
	AD	$4,$30,WRT01	;$4 = $4 + 1

WRT02:	LDW	$4,$18		;
	AN	$4,&H0C		;$4 = Y address * 4
	BIU	$4		;$4 = $4 * 2
	ADW	$20,$4
	INVW	$2
	ANW	$20,$2		;clear the digit
	ORW	$20,$0		;insert input number
	SBW	$20,$4

	SBC	$18,15		;check scroll
	JR	Z,RIG01		;
	AD	$18,$30
	JP	DUMP
;
;	write data
;
WRT03:	PRE	IZ,$20		;load binary data
	LDD	$1,(IZ+$18)	;
	SBC	$23,$31		;check cursor digit flag
	JR	NZ,WRT04	;
	DIU	$0		;upper digit
	AN	$1,&H0F,WRT05	;lower digit
WRT04:	AN	$1,&HF0		;
WRT05:	OR	$1,$0		;merge
	ST	$1,(IZ+$31)	;write input data
	CAL	DUMP1		;1 byte dump
	JR	RIG02		;
;
;	move cursor to right
;
RIGHT:	SBC	$22,$31		;check input mode flag
	JR	NZ,RIG02	;
;
;	address input mode
;
	AD	$18,$30		;$18 = $18 + 1
	SBC	$18,16		;
	RTN	C		;check scroll
RIG01:	LD	$18,12,ROLUP	;top of line 3rd.
;
;	data input mode
;
RIG02:	XR	$23,$30		;check cursor digit flag
	RTN	NZ		;
RIG03:	AD	$18,$30		;$18 = $18 + 1
	SBC	$18,32		;
	RTN	C		;scroll check
RIG04:	LD	$18,24,ROLUP	;top of line 3rd.
;
;	move cursor to left
;
LEFT:	SBC	$22,$31		;check input mode flag
	JR	NZ,LFT01	;
;
;	address input mode
;
	LD	$0,3,LFT02
;
;	data input mode
;
LFT01:	XR	$23,$30		;change cursor digit flag
	RTN	Z		;
	LD	$0,7
LFT02:	SB	$18,$30		;$18 = $18 - 1
	RTN	NC		;check scroll
	LD	$18,$0,ROLDN	;end of line 1st.
;
;	up cursor
;
UP:	SBC	$22,$31		;check input mode flag
	JR	NZ,UP01		;
;
;	address input mode
;
	LD	$0,4,UP02	;$18 = $18 - 4
;
;	data input mode
;
UP01:	LD	$0,8		;$18 = $18 - 8
UP02:	SB	$18,$0		;
	RTN	NC		;check scroll
	AD	$18,$0		;
ROLDN:	SBW	$20,$14,DUMP	;roll down (8byte)
;
;	down cursor
;
DOWN:	SBC	$22,$31		;check input mode flag
	JR	NZ,DWN01	;
;
;	address input mode
;
	SBC	$18,12		;check scroll
	JR	NC,ROLUP	;
	AD	$18,4		;$18 = $18 + 4
	RTN
;
;	data input mode
;
DWN01:	SBC	$18,24		;check scroll
	JR	NC,ROLUP	;
	AD	$18,8		;$18 = $18 + 8
	RTN
;
;	reset cursor position
;
CURRE:	SBC	$22,$31		;check input mode flag
	RTN	Z		;
	LDW 	$0,$18
	AN	$0,&H07
	AN	$18,&H18
	ADW	$20,$0,DUMP
;
;	reset address
;
ADRRE:	LD	$0,$20
	AN	$20,&HF8
	SBC	$22,$31		;check input mode flag
	JR	Z,DUMP		;
	SB	$0,$20
	AD	$18,$0
	SBC	$18,32		;scroll check
	JR	C,DUMP		;
	SB	$18,8		;$18 = $18 - 8

ROLUP:	ADW	$20,$14,DUMP	;roll up (8byte)

SCNUP:	ADW	$20,$16,DUMP	;screen up   (32byte)
SCNDN:	SBW	$20,$16,DUMP	;screen down (32byte)
;
;	change bank
;
BANKC:	AD	$24,$30		;bank + 1
	AN	$24,&H03	;$24 = 4 -> 0
SETBK:	LD	$2,$24
	DIU	$2		;x 8
	BIU	$2		;x 2
	BIU	$2		;x 2
	OR	$2,&H15
	PST	UA,$2,DUMP	;set bank of IZ
;
;	clear memory
;
CLM:	SBC	$22,$31		;check input mode flag
	RTN	Z		;
	PRE	IZ,$20		;set start address
	LD	$1,32		;32 byte
CLM01:	STI	$31,(IZ+$31)	;store null
	SB	$1,$30		;counter - 1
	JR	NZ,CLM01
;
;	dump 32bytes binary data
;
DUMP:	PRE	IZ,$20
	PRE	IX,EDTOP
	LD	$10,4		;set line counter

DMP01:	LD	$0,$24		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STI	$1,(IX+$31)	;output bank number

	GRE	IZ,$3		;$3,4 = address
	LD	$0,$4		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$31)	;output address (upper)

	LD	$0,$3		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$31)	;output address (lower)

	LDW	$6,&H0800	;reset counter
DMP02:	LDI	$0,(IZ+$31)	;load binary data
	AD	$6,$0		;add check-sum counter
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$30)	;output data
	SB	$7,$30		;counter - 1
	JR	NZ,DMP02	;

	LD	$0,$6		;
	CAL	CNASC		;$0 binary -> ascii $0,$1
	STIW	$0,(IX+$30)	;output check-sum

	SB	$10,$30		;line counter - 1
	JR	NZ,DMP01	;
;
;	display data in LCD (character mode)
;
	PRE	IX,EDTOP	;left side
	LD	$0,&H83		;set character output mode
	LDW	$1,&H03DE	;x=222, y=3 (for 1 line shift)
	CAL	DMP03

	LDD	$0,(IX+&H10)	;right side (IX=EDTOP+&H10)
	LD	$0,&H93		;set character output mode
DMP03:	PPO	&HC7		;set LCD command
	STLM	$0,3		;
	PPO	&HC6
	LD	$0,$31		;reset counter
DMP04:	LD	$3,(IX+$0)	;load ascii data
	CAL	SWAP		;swap $3 and output to LCD
	AD	$0,$30		;counter + 1
	ANC	$0,&H10		;
	JR	Z,DMP05		;
	AD	$0,&H10		;add offset
DMP05:	SBC	$0,&H80		;check end
	JR	NZ,DMP04	;
	RTN
;
;	exchange cursor position
;
EXCSR:	XR	$22,$30		;check input mode flag
	JR	NZ,EXC01	;
;
;	data input mode
;
	AN	$18,&H18
	BID	$18
	LD	$23,$31
	RTN
;
;	address input mode
;
EXC01:	AN	$18,&H0C
	BIU	$18
	RTN
;
;	output ascii character
;
ASCII:	LDW	$7,&H8301	;set character output mode
	LDW	$9,&H02DE	;x=222, y=2 (for 1 line shift)
	JR	OUTDT
;
;	output bitmap data
;
BITMP:	LDW	$7,&H8202	;set binary output mode
	LDW	$9,&H0300	;x=0, y=3

OUTDT:	PRE	IZ,$20
	CAL	OUTSB		;left side
	OR	$8,&H10		;
	CAL	OUTSB		;right side

	PST	IA,&H9		;
OUT01:	GRE	KY,$0		;check key hold
	SBC	$7,$0		;
	JR	Z,OUT01		;erase output data
	JP	DUMP

OUTSB:	PPO	&HC7		;
	STLM	$8,3		;set LCD command
	PPO	&HC6		;
	LD	$6,16		;set counter
	SBC	$7,$30		;check binary/ascii
	JR	Z,OUT03		;
OUT02:	LDIM	$0,(IZ+$31),6	;load binary data
	STLM	$0,6		;output to LCD
	SB	$6,$30		;counter - 1
	JR	NZ,OUT02
	RTN
OUT03:	LDI	$3,(IZ+$31)	;load data
	CAL	SWAP		;swap $3 and output to LCD
	SB	$6,$30		;counter - 1
	JR	NZ,OUT03
	RTN
;
;	dump 1 byte binary data
;
DUMP1:	PRE	IZ,$20		;load binary data
	LD	$1,(IZ+$18)	;
	SBC	$23,$31		;check cursor digit
	JR	NZ,DMP06	;
	DID	$1,DMP07
DMP06:	AN	$1,&H0F
DMP07:	CAL	CNA02		;binary -> ascii ($1)

	LD	$10,$26

	PRE	IX,EDTOP	;$1 -> text buffer
	ST	$1,(IX+$10)	;
	CAL	DMP09
;
;	check-sum area
;
	PRE	IZ,$20
	LD	$0,$18
	AN	$0,&H18
	LD	$1,7		;set counter
	LDI	$0,(IZ+$0)
DMP08:	LDI	$2,(IZ+$31)
	AD	$0,$2		;add check-sum counter
	SB	$1,$30		;counter - 1
	JR	NZ,DMP08
	CAL	CNASC		;$0 binary -> ascii $0,$1

	OR	$10,&H1F	;set cursor pointer to check-sum area
	SB	$10,$30		;

	PRE	IX,EDTOP	;$0,$1 -> EDTOP buffer
	STW	$0,(IX+$10)	;
	CAL	DMP10		;output $0 (upper digit)

	AD	$10,$30		;

DMP09:	LD	$0,$1		;output $1 (lower digit)
DMP10:	LD	$8,&H83		;set command
	CAL	LCDXY		;calculate X,Y address
	LD	$3,$0
	CAL	SWAP		;swap and output to LCD
	RTN
;
;	convert binary to ascii
;
CNASC:	LD	$1,$0
	DID	$0		;upper digit
	AD	$0,"0"		;
	SBC	$0,&H3A		;
	JR	C,CNA01		;check 0-9 or A-F
	AD	$0,7		;
CNA01:	AN	$1,&H0F		;lower digit
CNA02:	AD	$1,"0"		;
	SBC	$1,&H3A		;
	RTN	C		;check 0-9 or A-F
	AD	$1,7		;
	RTN
;
;	swap $3 and output to LCD
;
SWAP:	LD	$4,$3		;
	DIU	$4		;swap digit
	DID	$3		;
	OR	$3,$4		;
	STL	$3		;output character to LCD
	RTN
;
;	input key
;
@KYIN:	CAL	CSRST		;erase cursor
	LDW	$0,KYSTA	;key repeat on
	ST	&H06,($0)	;
	LDW	$28,KYIN	;key scan
	CAL	RCS		;
;
;	cursor set/reset
;
CSRST:	LD	$10,$26		;
	LD	$8,&H22		;xor mode
	CAL	LCDXY		;calculate X-Y address
	LD	$29,$30		;
	STLW	$29		;set cursor bar
	STLW	$29		;
	STLW	$29		;
	RTN
;
;	calulate LCD X-Y address
;
LCDXY:	LD	$4,$10		;$4 : Y position
	DID	$4		;$4 = $4 /  8
	BID	$4		;$4 = $4 / 16

	LD	$6,$8		;$6 : binary/ascii flag
	AN	$6,$30		;
	JR	Z,LCD00		;

	LD	$5,$10		;check special position
	AN	$5,&H1F		;
	SBC	$5,&H18		;x=24
	JR	Z,LCD01		;
	SBC	$5,&H10		;x=16
	JR	Z,LCD02		;

LCD00:	LD	$3,$10		;$3 : X position within the block
	AN	$3,&H07		;
	LD	$5,$3		;
	BIU	$5		;$5<- $3 *  2
	AD	$3,$5		;$3 = $3 *  3
	BIU	$3		;$3 = $3 *  6
	SB	$3,$6		;$3 = $3 - offset
	BIU	$3		;$3 = $3 * 12

	LD	$5,$10		;$5 = check block 1/2
	AN	$5,&H08		;
	DIU	$5		;$5 = $5 *  8

	OR	$3,$5,LCD03	;$3 = $3 + $5

LCD01:	LD	$3,&H5E,LCD03	;modify X address for x=24

LCD02:	LD	$3,&HDE		;
	SB	$4,$30		;modify X,Y address for x=16
	JR	NC,LCD03	;
	LD	$4,3

LCD03:	LD	$2,$10		;$2 : LCD command
	AN	$2,&H10		;check LCD1/LCD2
	OR	$2,$8		;
	PPO	&HC7		;
	STLM	$2,3		;set LCD command
	PPO	&HC6		;
	RTN
;
;	BIOS call
;
RCS:	PST	UA,&H54
	JP	BIOS
;
;	jump address and key data table
;
JPTBL:	DB	&H1C		; ""
	DW	RIGHT

	DB	&H1D		; ""
	DW	LEFT

	DB	&H1E		; ""  
	DW	UP

	DB	&H1F		; ""
	DW	DOWN

	DB	"+"		; "+"
	DW	SCNUP

	DB	"-"		; "-"
	DW	SCNDN

	DB	"*"		; "*"
	DW	EXCSR

	DB	"/"		; "/"
	DW	BANKC

	DB	&HE0		; "ENG"
	DW	ADRRE

	DB	&HE9		; "ANS"
	DW	CURRE

	DB	"."		; "."
	DW	BITMP

	DB	&HA0		; "|E"
	DW	ASCII

	DB	&H0C		; "CLS"
	DW	CLM